#!/usr/bin/perl
#
# set CMOS bits to hop on BTO and rebuild the box.


use strict;
use lib qw(/usr/sausalito/perl);
use BTO;
use Build;

# constants
my $nvramBIN = "/sbin/nvram";
my $autobuildLatestFlag = 0;

die "Usage:\n$0 <product type> <product> <version> <ip of BTO>\n" if $#ARGV lt 2;
my ($productType, $product, $version, $BTOIP) = @ARGV;

# set our BTOIP cmos bits
if (defined($BTOIP)) {
  if(! setBTOIP($BTOIP)) {
    print "Could not set the BTO IP in cmos.  Not root?\n";
    exit 1;
  }
}
# if version is set to "LATEST" we want the latest build of that product
if ($version =~ /latest/i) {
  $autobuildLatestFlag = ($version =~ /auto/i) ? 1 : 0;
  $version = returnLatest($productType, $product);
  if ($version eq "-1") {
    print "We have the latest version for $productType: $product\n";
    exit 0;
  }
}

# set our BTO flags
if(!setBTOflags_nvram($productType, $product, $version, $autobuildLatestFlag)) {
  print "Could not set BTO flags, invalid product type or product?\n";
  clearBTOflags();
} else {
  # set to netboot
  system("$nvramBIN -c defboot n");
  if ($autobuildLatestFlag) {
    print "Rebooting for autobuild latest config";
  } else {
    print "Rebooting to build $productType: $product $version off of BTO: $BTOIP\n";
  }
  sleep 3;
  system("/sbin/reboot");
}

#----------------------------------END--------------------------------------#

######################################################
#
# Takes in type, product and returns the latest version
# or -1 if there are no later versions 
#
sub returnLatest {
  my ($type, $product) = @_;
  my $btoIP = join(".", getBTOIP_cmos());
  my $mountDir = "/mnt";
  my $btoDir = "$mountDir/bto-sync/$type";
  
  my $myVersionCVS = get_product();
  my $latestVersion;
  my @versions;
  
  # Mount the BTO server
  system("mount -t nfs $btoIP:/nfsroot-x86 $mountDir");
  if ($? != 0) {
    print "Could not mount $btoIP!\n"; 
    exit 0;
  }
  # Get our list of versions
  opendir(DIR, $btoDir);
  while (my $build = readdir(DIR)) {
    my $build_version;
    if ($build =~ /^$product/) {
      if ($build =~ /_(.+)/) { $build_version = $1;}
      push @versions, $build_version;
    }
  }
  closedir(DIR);
  
  @versions = sort versions @versions;
  $latestVersion = pop @versions;
  
  # compare to see if this version is better than what we have
  my $latestVersionCVS = getCVSversion("$btoDir/$product\_$latestVersion/installer/release_spec.$product");
  system("umount $mountDir");
  if (($latestVersionCVS ne $myVersionCVS) & isGreater($latestVersionCVS, $myVersionCVS)) {
    return $latestVersion;
  } else {
    return -1;
  }
}

######################################################
#
# Returns my build version and product type
#  
sub get_product {
  my ($build, $product);
  if (open(BUILD, '/etc/build')) {
    while (<BUILD>) {
      if (/build\s+(\S+)/) {
	$build = $1;
      }
      if (/for\s+a\s+(\S+)\sin/) {
	$product = $1;
      }
    }
    close(BUILD);
  }
  return ($product, $build);
}

