#!/usr/bin/perl

use strict;
use lib qw(/usr/sausalito/perl);
use File::Copy;
use FileHandle;
use Devel;

#############################################################
#
#  Variables and options setup
#
#############################################################
my $PRODUCT = "";
my $CVSTAG = "";
my $BUILD_DIR = "/home/build";
my $HELP = 0;
my $QUIET = 0;
my %MODULE_LIST;

use Getopt::Long;
GetOptions( 
      	"cvs-tags=s" => \$CVSTAG,
	"build-dir=s" => \$BUILD_DIR,
	"product=s" => \$PRODUCT,
	"help"    => \$HELP,
	"quiet"   => \$QUIET);

if ($HELP || ($PRODUCT eq "")) {
  print STDERR <<EOT ;
You can specify these options:
  --product=                  <product name>
  --build-dir=                <checkout directory> (defaults to "/home/build")
  --cvs-tags=                 <cvs tags to pass in module checkout>  (optional)
  --quiet                     <only display output on errors>
  --help                      <this text>
EOT
  exit(1);
}

# check environment variables
if (!$ENV{"CVSROOT"}) {
  print STDERR <<EOT ;
Error: all of these environment variables must be defined:
  CVSROOT
EOT
  exit(1);
}


# make sure checkout directory exists.
if (!-d $BUILD_DIR) {
  mkdir($BUILD_DIR, 0755) || die "Can't make $BUILD_DIR\n";
}
chdir($BUILD_DIR);

$QUIET || print "*** checking out all modules for $PRODUCT ***\n";
$QUIET || print "Checking out products.prd\n";
if (-d 'products.prd') {
  if (!cvs_cmd("update -PAd products.prd")) {
    die "Error on products.prd update.\n";
  }
}else{
  if (!cvs_cmd("co products.prd")) {
    die "Error on products.prd checkout.\n";
  }
}

if (-f "products.prd/$PRODUCT/devel_list"){
  check_out_modules("products.prd/$PRODUCT/devel_list", $CVSTAG, \%MODULE_LIST, $QUIET) ||
    die "Error on checkout of modules\n";
}

# Checkout all of the modules here.
opendir PRODUCT, "products.prd/$PRODUCT/" ||
  die "Can't open products.prd/$PRODUCT/, unable to access packing_list files\n";
foreach my $list (grep /^packing_list/o, readdir PRODUCT) {
  check_out_modules("products.prd/$PRODUCT/$list", $CVSTAG, \%MODULE_LIST, $QUIET) || 
    die "Error on checkout of modules\n";
}
closedir PRODUCT;

# Success
$QUIET || print "End of checkout for $PRODUCT\n";
exit 0;
