#!/usr/bin/perl -w

#####################################################
#
# Takes release_spec file
# and directory with RPM's and verifies that the 
# RPM's exist.

use strict;
use File::Copy;

if ($#ARGV != 1) {
  die "Usage:\n$0 <release_spec> <RPM directory>\n";
}

my ($RELSPEC, $RPMDIR) = @ARGV;

my $oldRPMDIR = "/fargo/i386";
my $exitValue = 0;

print "Verifying $RELSPEC with $RPMDIR\n";

# Get hardware generations supported
my @HARDWARE_GENS = split(/\n/, `grep -v '^#' $RELSPEC | sed -n -e "/%begin Hardware_Gen/,/%end Hardware_Gen/ p" | grep -v '^%'`);

open(LIST, "$RELSPEC") || die "Can't access $RELSPEC\n";
while (<LIST>) {
  my $dir = "";
  my $file = "";
  
  if (/\#?(RPM:)?\s?(.*\.rpm)/) {
    my $file = $2;
    chomp $file;
    # Look to see if there is a tag specifying which module the RPM exists in
    if (/:(\S+)/ && (!($1 eq "."))) {
      $dir = "$RPMDIR/$1";
    } else {
      $dir = "$RPMDIR/rpms";
    }
    # Look to see if there is a hardware tag on the RPM
    if ($file =~ /\%HGEN\%/) {
      foreach my $HARDWARE_GEN (@HARDWARE_GENS) {
	my $realfile = $file;
	$realfile =~ s/\%HGEN\%/$HARDWARE_GEN/g;
	if (! -e "$dir/$realfile") {
	  print "Hardware specific $file missing in $dir from $RELSPEC\n";
	  $exitValue = 1;
	}
      }
    }
    # No hardware tag
    elsif (! -e "$dir/$file") {
      # We are missing the file, check in old repository.
      if (-e "$oldRPMDIR/$file") {
	# it exists, lets copy it over to new dir
	print "$file missing in $dir from $RELSPEC but exists in $oldRPMDIR, so copying file to $dir\n";
	if (! (copy("$oldRPMDIR/$file", "$dir") && chmod(0644, "$dir/$file"))) {
	  print "copy of $file to $dir failed\n";
	}
      } else {
	# it doesn't exist, cache an error value
	print "$file missing in $dir from $RELSPEC\n";
	$exitValue = 1;
      }
    }
  }
}
close(LIST);

if (!$exitValue) {
  print "$RELSPEC verified\n";
}

exit $exitValue;

